@@ -1,9 +1,12 @@
-import pytest
+import pickle
 from typing import Union
+
+import numpy as np
 import torch
 import torchvision
-import pickle
+import pytest
 from pytest_lazyfixture import lazy_fixture
+
 from ..fixtures import *
 from ...quantus.helpers import *
 
@@ -20,14 +23,12 @@ if util.find_spec("zennit"):
     zennit_tests = [
         # Zennit
         (
-            lazy_fixture("load_mnist_model"),
-            lazy_fixture("load_mnist_images"),
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "canonizer": None,
                 "composite": None,
                 "attributor": zattr.Gradient,
-                "img_size": 28,
-                "nr_channels": 1,
                 "normalise": True,
                 "xai_lib": "zennit",
             },
@@ -36,17 +37,51 @@ if util.find_spec("zennit"):
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
+            {
+                "canonizer": None,
+                "composite": None,
+                "attributor": zattr.Gradient,
+                "normalise": True,
+                "xai_lib": "zennit",
+            },
+            {"min": -1},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "canonizer": ztv.SequentialMergeBatchNorm,
                 "composite": zcomp.EpsilonPlus,
                 "attributor": zattr.Gradient,
-                "img_size": 28,
-                "nr_channels": 1,
                 "abs": True,
                 "xai_lib": "zennit",
             },
             {"min": 0},
         ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "canonizer": ztv.SequentialMergeBatchNorm,
+                "composite": zcomp.EpsilonPlus,
+                "attributor": zattr.Gradient,
+                "abs": True,
+                "xai_lib": "zennit",
+            },
+            {"min": 0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "canonizer": None,
+                "composite": "epsilon_alpha2_beta1_flat",
+                "attributor": zattr.Gradient,
+                "pos_only": True,
+                "xai_lib": "zennit",
+            },
+            {"min": 0},
+        ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
@@ -54,13 +89,23 @@ if util.find_spec("zennit"):
                 "canonizer": None,
                 "composite": "epsilon_alpha2_beta1_flat",
                 "attributor": zattr.Gradient,
-                "img_size": 28,
-                "nr_channels": 1,
                 "pos_only": True,
                 "xai_lib": "zennit",
             },
             {"min": 0},
         ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "canonizer": None,
+                "composite": "guided_backprop",
+                "attributor": zattr.Gradient,
+                "neg_only": True,
+                "xai_lib": "zennit",
+            },
+            {"max": 0},
+        ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
@@ -68,8 +113,6 @@ if util.find_spec("zennit"):
                 "canonizer": None,
                 "composite": "guided_backprop",
                 "attributor": zattr.Gradient,
-                "img_size": 28,
-                "nr_channels": 1,
                 "neg_only": True,
                 "xai_lib": "zennit",
             },
@@ -87,24 +130,26 @@ else:
     + [
         # Captum
         (
-            lazy_fixture("load_mnist_model"),
-            lazy_fixture("load_mnist_images"),
-            {"method": "Saliency", "img_size": 28, "nr_channels": 1},
-            {"min": 0.0},
-        ),
-        (
-            lazy_fixture("load_mnist_model"),
-            lazy_fixture("load_mnist_images"),
-            {"method": "GradientShap", "img_size": 28, "nr_channels": 1, "abs": True},
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "Saliency",
+            },
             {"min": 0.0},
         ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
             {
-                "method": "IntegratedGradients",
-                "img_size": 28,
-                "nr_channels": 1,
+                "method": "Saliency",
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "GradientShap",
                 "abs": True,
             },
             {"min": 0.0},
@@ -112,22 +157,71 @@ else:
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
-            {"method": "InputXGradient", "img_size": 28, "nr_channels": 1, "abs": True},
+            {
+                "method": "GradientShap",
+                "abs": True,
+            },
             {"min": 0.0},
         ),
         (
-            lazy_fixture("load_mnist_model"),
-            lazy_fixture("load_mnist_images"),
-            {"method": "Occlusion", "img_size": 28, "nr_channels": 1, "abs": True},
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "IntegratedGradients",
+                "abs": True,
+            },
             {"min": 0.0},
         ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
+            {
+                "method": "IntegratedGradients",
+                "abs": True,
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "InputXGradient",
+                "abs": True,
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "method": "InputXGradient",
+                "abs": True,
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "Occlusion",
+                "abs": True,
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "method": "Occlusion",
+                "abs": True,
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "method": "FeatureAblation",
-                "img_size": 28,
-                "nr_channels": 1,
                 "neg_only": True,
             },
             {"max": 0.0},
@@ -135,10 +229,17 @@ else:
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
+            {
+                "method": "FeatureAblation",
+                "neg_only": True,
+            },
+            {"max": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "method": "GradCam",
-                "img_size": 28,
-                "nr_channels": 1,
                 "gc_layer": "model._modules.get('conv_2')",
                 "abs": True,
             },
@@ -147,10 +248,18 @@ else:
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
+            {
+                "method": "GradCam",
+                "gc_layer": "model._modules.get('conv_2')",
+                "abs": True,
+            },
+            {"min": 0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "method": "Control Var. Sobel Filter",
-                "img_size": 28,
-                "nr_channels": 1,
                 "neg_only": True,
             },
             {"max": 0.0},
@@ -158,22 +267,53 @@ else:
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
-            {"method": "Gradient", "img_size": 28, "nr_channels": 1, "pos_only": True},
-            {"min": 0.0},
+            {
+                "method": "Control Var. Sobel Filter",
+                "neg_only": True,
+            },
+            {"max": 0.0},
         ),
         (
-            lazy_fixture("load_mnist_model"),
-            lazy_fixture("load_mnist_images"),
-            {"method": "Gradient", "img_size": 28, "nr_channels": 1, "abs": True},
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "Gradient",
+                "pos_only": True,
+            },
             {"min": 0.0},
         ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
+            {
+                "method": "Gradient",
+                "pos_only": True,
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "Gradient",
+                "abs": True,
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "method": "Gradient",
+                "abs": True,
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "method": "Control Var. Constant",
-                "img_size": 28,
-                "nr_channels": 1,
                 "constant_value": 0.0,
             },
             {"value": 0.0},
@@ -181,47 +321,75 @@ else:
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
+            {
+                "method": "Control Var. Constant",
+                "constant_value": 0.0,
+            },
+            {"value": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "method": "Gradient",
                 "normalise": True,
                 "normalise_func": normalise_by_negative,
-                "img_size": 28,
-                "nr_channels": 1,
             },
             {"min": -1.0, "max": 1.0},
         ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
+            {
+                "method": "Gradient",
+                "normalise": True,
+                "normalise_func": normalise_by_negative,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "method": "Gradient",
                 "normalise": True,
                 "abs": True,
                 "normalise_func": normalise_by_max,
-                "img_size": 28,
-                "nr_channels": 1,
             },
             {"min": 0.0, "max": 1.0},
         ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
-            {"method": "Saliency", "img_size": 28, "nr_channels": 1},
-            {"min": 0.0},
+            {
+                "method": "Gradient",
+                "normalise": True,
+                "abs": True,
+                "normalise_func": normalise_by_max,
+            },
+            {"min": 0.0, "max": 1.0},
         ),
         (
-            lazy_fixture("load_mnist_model"),
-            lazy_fixture("load_mnist_images"),
-            {"method": "GradientShap", "img_size": 28, "nr_channels": 1, "abs": True},
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "Saliency",
+            },
             {"min": 0.0},
         ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
             {
-                "method": "IntegratedGradients",
-                "img_size": 28,
-                "nr_channels": 1,
+                "method": "Saliency",
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "GradientShap",
                 "abs": True,
             },
             {"min": 0.0},
@@ -229,22 +397,71 @@ else:
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
-            {"method": "InputXGradient", "img_size": 28, "nr_channels": 1, "abs": True},
+            {
+                "method": "GradientShap",
+                "abs": True,
+            },
             {"min": 0.0},
         ),
         (
-            lazy_fixture("load_mnist_model"),
-            lazy_fixture("load_mnist_images"),
-            {"method": "Occlusion", "img_size": 28, "nr_channels": 1, "abs": True},
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "IntegratedGradients",
+                "abs": True,
+            },
             {"min": 0.0},
         ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
+            {
+                "method": "IntegratedGradients",
+                "abs": True,
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "InputXGradient",
+                "abs": True,
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "method": "InputXGradient",
+                "abs": True,
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "Occlusion",
+                "abs": True,
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "method": "Occlusion",
+                "abs": True,
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "method": "FeatureAblation",
-                "img_size": 28,
-                "nr_channels": 1,
                 "neg_only": True,
             },
             {"max": 0.0},
@@ -252,10 +469,17 @@ else:
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
+            {
+                "method": "FeatureAblation",
+                "neg_only": True,
+            },
+            {"max": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "method": "GradCam",
-                "img_size": 28,
-                "nr_channels": 1,
                 "gc_layer": "model._modules.get('conv_2')",
                 "abs": True,
             },
@@ -264,10 +488,18 @@ else:
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
+            {
+                "method": "GradCam",
+                "gc_layer": "model._modules.get('conv_2')",
+                "abs": True,
+            },
+            {"min": 0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "method": "Control Var. Sobel Filter",
-                "img_size": 28,
-                "nr_channels": 1,
                 "neg_only": True,
             },
             {"max": 0.0},
@@ -275,22 +507,53 @@ else:
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
-            {"method": "Gradient", "img_size": 28, "nr_channels": 1, "pos_only": True},
-            {"min": 0.0},
+            {
+                "method": "Control Var. Sobel Filter",
+                "neg_only": True,
+            },
+            {"max": 0.0},
         ),
         (
-            lazy_fixture("load_mnist_model"),
-            lazy_fixture("load_mnist_images"),
-            {"method": "Gradient", "img_size": 28, "nr_channels": 1, "abs": True},
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "Gradient",
+                "pos_only": True,
+            },
             {"min": 0.0},
         ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
+            {
+                "method": "Gradient",
+                "pos_only": True,
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "Gradient",
+                "abs": True,
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "method": "Gradient",
+                "abs": True,
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "method": "Control Var. Constant",
-                "img_size": 28,
-                "nr_channels": 1,
                 "constant_value": 0.0,
             },
             {"value": 0.0},
@@ -298,15 +561,43 @@ else:
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
+            {
+                "method": "Control Var. Constant",
+                "constant_value": 0.0,
+            },
+            {"value": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "method": "Gradient",
                 "normalise": True,
                 "normalise_func": normalise_by_negative,
-                "img_size": 28,
-                "nr_channels": 1,
             },
             {"min": -1.0, "max": 1.0},
         ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "method": "Gradient",
+                "normalise": True,
+                "normalise_func": normalise_by_negative,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "Gradient",
+                "normalise": True,
+                "abs": True,
+                "normalise_func": normalise_by_max,
+            },
+            {"min": 0.0, "max": 1.0},
+        ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
@@ -315,8 +606,6 @@ else:
                 "normalise": True,
                 "abs": True,
                 "normalise_func": normalise_by_max,
-                "img_size": 28,
-                "nr_channels": 1,
             },
             {"min": 0.0, "max": 1.0},
         ),
@@ -324,13 +613,19 @@ else:
         (
             lazy_fixture("load_mnist_model_tf"),
             lazy_fixture("load_mnist_images_tf"),
-            {"method": "Gradient", "abs": True, "img_size": 28, "nr_channels": 1},
+            {
+                "method": "Gradient",
+                "abs": True,
+            },
             {"min": 0.0, "max": 1.0},
         ),
         (
             lazy_fixture("load_mnist_model_tf"),
             lazy_fixture("load_mnist_images_tf"),
-            {"method": "Occlusion", "abs": True, "img_size": 28, "nr_channels": 1},
+            {
+                "method": "Occlusion",
+                "abs": True,
+            },
             {"min": 0.0, "max": 1.0},
         ),
         (
@@ -338,8 +633,6 @@ else:
             lazy_fixture("load_mnist_images_tf"),
             {
                 "method": "IntegratedGradients",
-                "img_size": 28,
-                "nr_channels": 1,
                 "abs": True,
             },
             {"min": 0.0},
@@ -347,19 +640,24 @@ else:
         (
             lazy_fixture("load_mnist_model_tf"),
             lazy_fixture("load_mnist_images_tf"),
-            {"method": "InputXGradient", "img_size": 28, "nr_channels": 1, "abs": True},
+            {
+                "method": "InputXGradient",
+                "abs": True,
+            },
             {"min": 0.0},
         ),
         (
             lazy_fixture("load_mnist_model_tf"),
             lazy_fixture("load_mnist_images_tf"),
-            {"img_size": 28, "nr_channels": 1},
+            {
+            },
             {"warning": UserWarning},
         ),
         (
             None,
             lazy_fixture("load_mnist_images_tf"),
-            {"img_size": 28, "nr_channels": 1},
+            {
+            },
             {"exception": ValueError},
         ),
     ],
@@ -405,18 +703,26 @@ def test_explain_func(
     "model,data,params,expected",
     [
         (
-            lazy_fixture("load_mnist_model"),
-            lazy_fixture("load_mnist_images"),
-            {"method": "Saliency", "img_size": 28, "nr_channels": 1},
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "Saliency",
+            },
             {"min": 0.0},
         ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
+            {
+                "method": "Saliency",
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "method": "Control Var. Constant",
-                "img_size": 28,
-                "nr_channels": 1,
                 "constant_value": 0.0,
             },
             {"value": 0.0},
@@ -424,8 +730,27 @@ def test_explain_func(
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
-            {"method": "GradCam", "img_size": 28, "nr_channels": 1},
-            {"exception": AssertionError},
+            {
+                "method": "Control Var. Constant",
+                "constant_value": 0.0,
+            },
+            {"value": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "GradCam",
+            },
+            {"exception": ValueError},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "method": "GradCam",
+            },
+            {"exception": ValueError},
         ),
     ],
 )
@@ -470,46 +795,103 @@ def test_generate_captum_explanation(
     "model,data,params,expected",
     [
         (
-            lazy_fixture("load_mnist_model_tf"),
-            lazy_fixture("load_mnist_images_tf"),
-            {"method": "Gradient", "img_size": 28, "nr_channels": 1},
-            {"min": 0.0},
+            lazy_fixture("load_1d_3ch_conv_model_tf"),
+            lazy_fixture("almost_uniform_1d_no_abatch_channel_last"),
+            {
+                "method": "Gradient",
+            },
+            {"exception": ValueError},
         ),
         (
             lazy_fixture("load_mnist_model_tf"),
             lazy_fixture("load_mnist_images_tf"),
-            {"method": "Occlusion", "img_size": 28, "nr_channels": 1},
+            {
+                "method": "Gradient",
+            },
             {"min": 0.0},
         ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model_tf"),
+            lazy_fixture("almost_uniform_1d_no_abatch_channel_last"),
+            {
+                "method": "Occlusion",
+            },
+            {"exception": IndexError},
+        ),
         (
             lazy_fixture("load_mnist_model_tf"),
             lazy_fixture("load_mnist_images_tf"),
-            {"method": "InputXGradient", "img_size": 28, "nr_channels": 1},
+            {
+                "method": "Occlusion",
+            },
             {"min": 0.0},
         ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model_tf"),
+            lazy_fixture("almost_uniform_1d_no_abatch_channel_last"),
+            {
+                "method": "InputXGradient",
+            },
+            {"exception": ValueError},
+        ),
         (
             lazy_fixture("load_mnist_model_tf"),
             lazy_fixture("load_mnist_images_tf"),
-            {"method": "IntegratedGradients", "img_size": 28, "nr_channels": 1},
+            {
+                "method": "InputXGradient",
+            },
             {"min": 0.0},
         ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model_tf"),
+            lazy_fixture("almost_uniform_1d_no_abatch_channel_last"),
+            {
+                "method": "IntegratedGradients",
+            },
+            {"exception": ValueError},
+        ),
         (
             lazy_fixture("load_mnist_model_tf"),
             lazy_fixture("load_mnist_images_tf"),
-            {"method": "GradCam", "img_size": 28, "nr_channels": 1},
-            {"exception": AssertionError},
+            {
+                "method": "IntegratedGradients",
+            },
+            {"min": 0.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model_tf"),
+            lazy_fixture("almost_uniform_1d_no_abatch_channel_last"),
+            {
+                "method": "GradCam",
+            },
+            {"exception": ValueError},
         ),
         (
             lazy_fixture("load_mnist_model_tf"),
             lazy_fixture("load_mnist_images_tf"),
             {
                 "method": "GradCam",
-                "img_size": 28,
-                "nr_channels": 1,
+            },
+            {"exception": ValueError},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model_tf"),
+            lazy_fixture("almost_uniform_1d_no_abatch_channel_last"),
+            {
+                "method": "GradCam",
                 "gc_layer": "dense_1",
             },
             {"exception": Exception},
         ),
+        (
+            lazy_fixture("load_mnist_model_tf"),
+            lazy_fixture("load_mnist_images_tf"),
+            {
+                "method": "GradCam",
+                "gc_layer": "dense_1",
+            },
+            {"exception": ValueError},
+        ),
     ],
 )
 def test_generate_tf_explanation(
@@ -555,13 +937,33 @@ def test_generate_tf_explanation(
         (
             lazy_fixture("load_mnist_model_tf"),
             lazy_fixture("load_mnist_images_tf"),
-            {"method": "Gradient", "img_size": 28, "nr_channels": 1},
+            {
+                "method": "Gradient",
+            },
             {"min": 0.0, "max": 1.0},
         ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model_tf"),
+            lazy_fixture("almost_uniform_1d_no_abatch_channel_last"),
+            {
+                "method": "Gradient",
+            },
+            {"exception": ValueError},
+        ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
-            {"method": "Gradient", "img_size": 28, "nr_channels": 1},
+            {
+                "method": "Gradient",
+            },
+            {"min": -3},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "Gradient",
+            },
             {"min": -3},
         ),
     ],
@@ -572,7 +974,13 @@ def test_get_explanation(
     params: dict,
     expected: Union[float, dict, bool],
 ):
-    x_batch, y_batch = (data["x_batch"], data["y_batch"])
+    x_batch, y_batch = data["x_batch"], data["y_batch"]
+
+    if "exception" in expected:
+        with pytest.raises(expected["exception"]):
+            a_batch = get_explanation(
+                model=model, inputs=x_batch, targets=y_batch, **params)
+        return
 
     a_batch = get_explanation(model=model, inputs=x_batch, targets=y_batch, **params)
 
