@@ -1,9 +1,14 @@
-import pytest
 from typing import Union
+
+import numpy as np
+import pytest
 from pytest_lazyfixture import lazy_fixture
+
 from ..fixtures import *
 from ...quantus.metrics import *
-from ...quantus.helpers.pytorch_model import PyTorchModel
+from ...quantus.helpers import *
+from ...quantus.helpers import perturb_func
+from ...quantus.helpers.explanation_func import explain
 
 
 @pytest.mark.faithfulness
@@ -21,8 +26,6 @@ from ...quantus.helpers.pytorch_model import PyTorchModel
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "max_steps_per_input": 2,
                 "disable_warnings": False,
                 "display_progressbar": False,
@@ -39,8 +42,6 @@ from ...quantus.helpers.pytorch_model import PyTorchModel
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": False,
                 "a_batch_generate": False,
@@ -57,8 +58,6 @@ from ...quantus.helpers.pytorch_model import PyTorchModel
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": True,
             },
@@ -75,8 +74,6 @@ from ...quantus.helpers.pytorch_model import PyTorchModel
                 "normalise": True,
                 "explain_func": explain,
                 "method": "IntegratedGradients",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": False,
             },
@@ -92,13 +89,75 @@ from ...quantus.helpers.pytorch_model import PyTorchModel
                 "normalise": True,
                 "explain_func": explain,
                 "method": "InputXGradient",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": False,
             },
             {"min": -1.0, "max": 1.0},
         ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_func": baseline_replacement_by_indices,
+                "perturb_baseline": "mean",
+                "explain_func": explain,
+                "nr_runs": 10,
+                "similarity_func": correlation_spearman,
+                "normalise": True,
+                "subset_size": 100,
+                "disable_warnings": False,
+                "display_progressbar": False,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d"),
+            {
+                "perturb_func": baseline_replacement_by_indices,
+                "perturb_baseline": "mean",
+                "nr_runs": 10,
+                "similarity_func": correlation_spearman,
+                "normalise": True,
+                "subset_size": 10,
+                "disable_warnings": True,
+                "display_progressbar": False,
+                "a_batch_generate": False,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d"),
+            {
+                "perturb_func": baseline_replacement_by_indices,
+                "perturb_baseline": "mean",
+                "nr_runs": 10,
+                "similarity_func": correlation_spearman,
+                "normalise": True,
+                "subset_size": 100,
+                "disable_warnings": True,
+                "display_progressbar": False,
+                "a_batch_generate": False,
+            },
+            {"exception": ValueError},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_func": baseline_replacement_by_indices,
+                "perturb_baseline": "mean",
+                "explain_func": explain,
+                "nr_runs": 10,
+                "similarity_func": correlation_spearman,
+                "normalise": True,
+                "subset_size": 784,
+                "disable_warnings": False,
+                "display_progressbar": False,
+            },
+            {"exception": ValueError},
+        ),
     ],
 )
 def test_faithfulness_correlation(
@@ -111,16 +170,30 @@ def test_faithfulness_correlation(
         data["x_batch"],
         data["y_batch"],
     )
-    explain = params["explain_func"]
     if params.get("a_batch_generate", True):
+        explain = params["explain_func"]
         a_batch = explain(
             model=model,
             inputs=x_batch,
             targets=y_batch,
             **params,
         )
+    elif "a_batch" in data:
+        a_batch = data["a_batch"]
     else:
         a_batch = None
+
+    if "exception" in expected:
+        with pytest.raises(expected["exception"]):
+            scores = FaithfulnessCorrelation(**params)(
+                model=model,
+                x_batch=x_batch,
+                y_batch=y_batch,
+                a_batch=a_batch,
+                **params,
+            )[0]
+        return
+
     scores = FaithfulnessCorrelation(**params)(
         model=model,
         x_batch=x_batch,
@@ -148,8 +221,6 @@ def test_faithfulness_correlation(
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "max_steps_per_input": 2,
                 "disable_warnings": False,
                 "display_progressbar": False,
@@ -166,8 +237,6 @@ def test_faithfulness_correlation(
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Gradient",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": False,
             },
@@ -184,8 +253,6 @@ def test_faithfulness_correlation(
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Gradient",
-                "img_size": 28,
-                "nr_channels": 1,
                 "max_steps_per_input": 2,
                 "disable_warnings": True,
                 "display_progressbar": False,
@@ -203,14 +270,27 @@ def test_faithfulness_correlation(
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "max_steps_per_input": 2,
                 "disable_warnings": True,
                 "display_progressbar": True,
             },
             {"min": -1.0, "max": 1.0},
         ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d"),
+            {
+                "perturb_func": baseline_replacement_by_indices,
+                "perturb_baseline": "uniform",
+                "features_in_step": 10,
+                "normalise": True,
+                "max_steps_per_input": 2,
+                "disable_warnings": False,
+                "display_progressbar": False,
+                "a_batch_generate": False,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
     ],
 )
 def test_faithfulness_estimate(
@@ -223,14 +303,16 @@ def test_faithfulness_estimate(
         data["x_batch"],
         data["y_batch"],
     )
-    explain = params["explain_func"]
     if params.get("a_batch_generate", True):
+        explain = params["explain_func"]
         a_batch = explain(
             model=model,
             inputs=x_batch,
             targets=y_batch,
             **params,
         )
+    elif "a_batch" in data:
+        a_batch = data["a_batch"]
     else:
         a_batch = None
     scores = FaithfulnessEstimate(**params)(
@@ -246,6 +328,118 @@ def test_faithfulness_estimate(
     ), "Test failed."
 
 
+@pytest.mark.faithfulness
+@pytest.mark.parametrize(
+    "model,data,params,expected",
+    [
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_baseline": "mean",
+                "segmentation_method": "slic",
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "max_steps_per_input": 2,
+                "disable_warnings": False,
+                "display_progressbar": False,
+            },
+            {"min": 0.0, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_baseline": "mean",
+                "segmentation_method": "slic",
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "abs": True,
+                "disable_warnings": True,
+                "display_progressbar": False,
+                "a_batch_generate": False,
+            },
+            {"min": 0.0, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_baseline": "mean",
+                "segmentation_method": "slic",
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "max_steps_per_input": 2,
+                "disable_warnings": True,
+                "display_progressbar": True,
+            },
+            {"min": 0.0, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d"),
+            {
+                "perturb_baseline": "mean",
+                "segmentation_method": "slic",
+                "normalise": True,
+                "max_steps_per_input": 2,
+                "disable_warnings": False,
+                "display_progressbar": False,
+                "a_batch_generate": False,
+            },
+            {"exception": ValueError},
+        ),
+    ],
+)
+def test_iterative_removal_of_features(
+    model,
+    data: np.ndarray,
+    params: dict,
+    expected: Union[float, dict, bool],
+):
+    x_batch, y_batch = (
+        data["x_batch"],
+        data["y_batch"],
+    )
+
+    if params.get("a_batch_generate", True):
+        explain = params["explain_func"]
+        a_batch = explain(
+            model=model,
+            inputs=x_batch,
+            targets=y_batch,
+            **params,
+        )
+    elif "a_batch" in data:
+        a_batch = data["a_batch"]
+    else:
+        a_batch = None
+
+    if "exception" in expected:
+        with pytest.raises(expected["exception"]):
+            scores = IterativeRemovalOfFeatures(**params)(
+                model=model,
+                x_batch=x_batch,
+                y_batch=y_batch,
+                a_batch=a_batch,
+                **params,
+            )
+        return
+
+    scores = IterativeRemovalOfFeatures(**params)(
+        model=model,
+        x_batch=x_batch,
+        y_batch=y_batch,
+        a_batch=a_batch,
+        **params,
+    )
+
+    assert scores is not None, "Test failed."
+
+
 @pytest.mark.faithfulness
 @pytest.mark.parametrize(
     "model,data,params,expected",
@@ -260,8 +454,6 @@ def test_faithfulness_estimate(
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "max_steps_per_input": 2,
                 "disable_warnings": False,
                 "display_progressbar": False,
@@ -278,8 +470,6 @@ def test_faithfulness_estimate(
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": False,
             },
@@ -295,8 +485,6 @@ def test_faithfulness_estimate(
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Gradient",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": False,
                 "a_batch_generate": False,
@@ -313,14 +501,27 @@ def test_faithfulness_estimate(
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "max_steps_per_input": 2,
                 "disable_warnings": True,
                 "display_progressbar": True,
             },
             {"allowed_dtypes": [True, False]},
         ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d"),
+            {
+                "perturb_func": baseline_replacement_by_indices,
+                "perturb_baseline": "black",
+                "features_in_step": 10,
+                "normalise": True,
+                "max_steps_per_input": 2,
+                "disable_warnings": False,
+                "display_progressbar": False,
+                "a_batch_generate": False,
+            },
+            {"allowed_dtypes": [True, False]},
+        ),
     ],
 )
 def test_monotonicity_arya(
@@ -333,14 +534,16 @@ def test_monotonicity_arya(
         data["x_batch"],
         data["y_batch"],
     )
-    explain = params["explain_func"]
     if params.get("a_batch_generate", True):
+        explain = params["explain_func"]
         a_batch = explain(
             model=model,
             inputs=x_batch,
             targets=y_batch,
             **params,
         )
+    elif "a_batch" in data:
+        a_batch = data["a_batch"]
     else:
         a_batch = None
     scores = MonotonicityArya(**params)(
@@ -371,8 +574,6 @@ def test_monotonicity_arya(
                 "similarity_func": correlation_kendall_tau,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "max_steps_per_input": 2,
                 "disable_warnings": False,
                 "display_progressbar": False,
@@ -393,8 +594,6 @@ def test_monotonicity_arya(
                 "similarity_func": correlation_kendall_tau,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "max_steps_per_input": 2,
                 "disable_warnings": True,
                 "display_progressbar": True,
@@ -402,6 +601,24 @@ def test_monotonicity_arya(
             },
             1.0,
         ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d"),
+            {
+                "eps": 1e-5,
+                "nr_samples": 10,
+                "features_in_step": 10,
+                "normalise": True,
+                "abs": True,
+                "perturb_baseline": "uniform",
+                "similarity_func": correlation_kendall_tau,
+                "max_steps_per_input": 2,
+                "disable_warnings": True,
+                "display_progressbar": False,
+                "a_batch_generate": False,
+            },
+            1.0,
+        ),
     ],
 )
 def test_monotonicity_nguyen(
@@ -414,14 +631,16 @@ def test_monotonicity_nguyen(
         data["x_batch"],
         data["y_batch"],
     )
-    explain = params["explain_func"]
     if params.get("a_batch_generate", True):
+        explain = params["explain_func"]
         a_batch = explain(
             model=model,
             inputs=x_batch,
             targets=y_batch,
             **params,
         )
+    elif "a_batch" in data:
+        a_batch = data["a_batch"]
     else:
         a_batch = None
     scores = MonotonicityNguyen(**params)(
@@ -449,11 +668,10 @@ def test_monotonicity_nguyen(
                 "explain_func": explain,
                 "method": "Saliency",
                 "abs": True,
-                "img_size": 28,
-                "nr_channels": 1,
                 "max_steps_per_input": 2,
                 "disable_warnings": False,
                 "display_progressbar": False,
+                "a_batch_generate": True,
             },
             {"min": 0.0, "max": 1.0},
         ),
@@ -466,10 +684,9 @@ def test_monotonicity_nguyen(
                 "normalise": False,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": False,
+                "a_batch_generate": True,
             },
             {"min": 0.0, "max": 1.0},
         ),
@@ -482,8 +699,7 @@ def test_monotonicity_nguyen(
                 "normalise": False,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
+                "a_batch_generate": True,
                 "disable_warnings": True,
                 "display_progressbar": False,
             },
@@ -498,8 +714,6 @@ def test_monotonicity_nguyen(
                 "normalise": False,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": False,
                 "a_batch_generate": False,
@@ -516,14 +730,25 @@ def test_monotonicity_nguyen(
                 "explain_func": explain,
                 "method": "Saliency",
                 "abs": True,
-                "img_size": 28,
-                "nr_channels": 1,
                 "max_steps_per_input": 2,
                 "disable_warnings": True,
                 "display_progressbar": True,
             },
             {"min": 0.0, "max": 1.0},
         ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d"),
+            {
+                "features_in_step": 10,
+                "normalise": False,
+                "perturb_func": perturb_func.baseline_replacement_by_indices,
+                "perturb_baseline": "mean",
+                "disable_warnings": True,
+                "a_batch_generate": False,
+            },
+            {"min": 0.0, "max": 1.0},
+        ),
     ],
 )
 def test_pixel_flipping(
@@ -536,14 +761,16 @@ def test_pixel_flipping(
         data["x_batch"],
         data["y_batch"],
     )
-    explain = params["explain_func"]
     if params.get("a_batch_generate", True):
+        explain = params["explain_func"]
         a_batch = explain(
             model=model,
             inputs=x_batch,
             targets=y_batch,
             **params,
         )
+    elif "a_batch" in data:
+        a_batch = data["a_batch"]
     else:
         a_batch = None
     scores = PixelFlipping(**params)(
@@ -573,10 +800,9 @@ def test_pixel_flipping(
                 "order": "morf",
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": False,
                 "display_progressbar": False,
+                "a_batch_generate": True,
             },
             {"min": -1, "max": 1.0},
         ),
@@ -590,8 +816,32 @@ def test_pixel_flipping(
                 "order": "random",
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
+                "disable_warnings": True,
+                "display_progressbar": False,
+                "a_batch_generate": False,
+            },
+            {"min": -1, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_baseline": "mean",
+                "patch_size": 7,
+                "normalise": True,
+                "order": "morf",
+                "disable_warnings": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "perturb_func": perturb_func.baseline_replacement_by_patch,
+                "a_batch_generate": False,
+            },
+            {"min": 0.0, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d"),
+            {
                 "disable_warnings": True,
                 "display_progressbar": False,
                 "a_batch_generate": False,
@@ -608,25 +858,6 @@ def test_pixel_flipping(
                 "order": "morf",
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
-                "disable_warnings": True,
-                "display_progressbar": False,
-            },
-            {"min": -1, "max": 1.0},
-        ),
-        (
-            lazy_fixture("load_mnist_model"),
-            lazy_fixture("load_mnist_images"),
-            {
-                "perturb_baseline": "mean",
-                "patch_size": 7,
-                "normalise": True,
-                "order": "morf",
-                "explain_func": explain,
-                "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": True,
             },
@@ -634,7 +865,7 @@ def test_pixel_flipping(
         ),
     ],
 )
-def test_region_segmentation(
+def test_region_perturbation(
     model,
     data: np.ndarray,
     params: dict,
@@ -644,14 +875,16 @@ def test_region_segmentation(
         data["x_batch"],
         data["y_batch"],
     )
-    explain = params["explain_func"]
     if params.get("a_batch_generate", True):
+        explain = params["explain_func"]
         a_batch = explain(
             model=model,
             inputs=x_batch,
             targets=y_batch,
             **params,
         )
+    elif "a_batch" in data:
+        a_batch = data["a_batch"]
     else:
         a_batch = None
     scores = RegionPerturbation(**params)(
@@ -678,8 +911,6 @@ def test_region_segmentation(
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "abs": True,
                 "max_steps_per_input": 2,
                 "disable_warnings": False,
@@ -696,8 +927,6 @@ def test_region_segmentation(
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": False,
                 "a_batch_generate": False,
@@ -713,8 +942,6 @@ def test_region_segmentation(
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Gradient",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": False,
             },
@@ -729,8 +956,6 @@ def test_region_segmentation(
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "abs": True,
                 "max_steps_per_input": 2,
                 "disable_warnings": True,
@@ -738,6 +963,21 @@ def test_region_segmentation(
             },
             {"min": 0.0, "max": 1.0},
         ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d"),
+            {
+                "perturb_baseline": "mean",
+                "patch_size": 7,
+                "normalise": True,
+                "abs": True,
+                "max_steps_per_input": 2,
+                "disable_warnings": False,
+                "display_progressbar": False,
+                "a_batch_generate": False,
+            },
+            {"min": 0.0, "max": 1.0},
+        ),
     ],
 )
 def test_selectivity(
@@ -750,14 +990,16 @@ def test_selectivity(
         data["x_batch"],
         data["y_batch"],
     )
-    explain = params["explain_func"]
     if params.get("a_batch_generate", True):
+        explain = params["explain_func"]
         a_batch = explain(
             model=model,
             inputs=x_batch,
             targets=y_batch,
             **params,
         )
+    elif "a_batch" in data:
+        a_batch = data["a_batch"]
     else:
         a_batch = None
     scores = Selectivity(**params)(
@@ -789,8 +1031,6 @@ def test_selectivity(
                 "explain_func": explain,
                 "method": "Saliency",
                 "abs": True,
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": False,
                 "display_progressbar": False,
             },
@@ -807,8 +1047,6 @@ def test_selectivity(
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": False,
             },
@@ -825,8 +1063,6 @@ def test_selectivity(
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Gradient",
-                "img_size": 28,
-                "nr_channels": 1,
                 "max_steps_per_input": 2,
                 "disable_warnings": True,
                 "display_progressbar": False,
@@ -846,13 +1082,27 @@ def test_selectivity(
                 "explain_func": explain,
                 "method": "Saliency",
                 "abs": True,
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": True,
             },
             {"min": -1.0, "max": 1.0},
         ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d"),
+            {
+                "perturb_baseline": "black",
+                "n_max_percentage": 0.9,
+                "features_in_step": 10,
+                "similarity_func": correlation_spearman,
+                "normalise": True,
+                "abs": True,
+                "disable_warnings": False,
+                "display_progressbar": False,
+                "a_batch_generate": False,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
     ],
 )
 def test_sensitivity_n(
@@ -866,14 +1116,16 @@ def test_sensitivity_n(
         data["y_batch"],
     )
 
-    explain = params["explain_func"]
     if params.get("a_batch_generate", True):
+        explain = params["explain_func"]
         a_batch = explain(
             model=model,
             inputs=x_batch,
             targets=y_batch,
             **params,
         )
+    elif "a_batch" in data:
+        a_batch = data["a_batch"]
     else:
         a_batch = None
     scores = SensitivityN(**params)(
@@ -887,94 +1139,3 @@ def test_sensitivity_n(
     assert all(
         ((s >= expected["min"]) & (s <= expected["max"])) for s in scores
     ), "Test failed."
-
-
-@pytest.mark.faithfulness
-@pytest.mark.parametrize(
-    "model,data,params,expected",
-    [
-        (
-            lazy_fixture("load_mnist_model"),
-            lazy_fixture("load_mnist_images"),
-            {
-                "perturb_baseline": "mean",
-                "segmentation_method": "slic",
-                "normalise": True,
-                "explain_func": explain,
-                "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
-                "max_steps_per_input": 2,
-                "disable_warnings": False,
-                "display_progressbar": False,
-            },
-            {"min": 0.0, "max": 1.0},
-        ),
-        (
-            lazy_fixture("load_mnist_model"),
-            lazy_fixture("load_mnist_images"),
-            {
-                "perturb_baseline": "mean",
-                "segmentation_method": "slic",
-                "normalise": True,
-                "explain_func": explain,
-                "method": "Saliency",
-                "abs": True,
-                "img_size": 28,
-                "nr_channels": 1,
-                "disable_warnings": True,
-                "display_progressbar": False,
-                "a_batch_generate": False,
-            },
-            {"min": 0.0, "max": 1.0},
-        ),
-        (
-            lazy_fixture("load_mnist_model"),
-            lazy_fixture("load_mnist_images"),
-            {
-                "perturb_baseline": "mean",
-                "segmentation_method": "slic",
-                "normalise": True,
-                "explain_func": explain,
-                "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
-                "max_steps_per_input": 2,
-                "disable_warnings": True,
-                "display_progressbar": True,
-            },
-            {"min": 0.0, "max": 1.0},
-        ),
-    ],
-)
-def test_iterative_removal_of_features(
-    model,
-    data: np.ndarray,
-    params: dict,
-    expected: Union[float, dict, bool],
-):
-    x_batch, y_batch = (
-        data["x_batch"],
-        data["y_batch"],
-    )
-    explain = params["explain_func"]
-
-    if params.get("a_batch_generate", True):
-        a_batch = explain(
-            model=model,
-            inputs=x_batch,
-            targets=y_batch,
-            **params,
-        )
-    else:
-        a_batch = None
-
-    scores = IterativeRemovalOfFeatures(**params)(
-        model=model,
-        x_batch=x_batch,
-        y_batch=y_batch,
-        a_batch=a_batch,
-        **params,
-    )
-
-    assert scores is not None, "Test failed."
