@@ -1,8 +1,13 @@
-import pytest
 from typing import Union
+
+import numpy as np
+import pytest
 from pytest_lazyfixture import lazy_fixture
+
 from ..fixtures import *
 from ...quantus.metrics import *
+from ...quantus.helpers import *
+from ...quantus.helpers.explanation_func import explain
 from ...quantus.helpers.model_interface import ModelInterface
 
 
@@ -11,21 +16,47 @@ from ...quantus.helpers.model_interface import ModelInterface
     "model,data,params,expected",
     [
         (
-            lazy_fixture("load_mnist_model"),
-            lazy_fixture("load_mnist_images"),
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "layer_order": "top_down",
                 "similarity_func": correlation_spearman,
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": False,
                 "display_progressbar": False,
             },
             {"min": -1.0, "max": 1.0},
         ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "layer_order": "top_down",
+                "similarity_func": correlation_spearman,
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "disable_warnings": False,
+                "display_progressbar": False,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "layer_order": "bottom_up",
+                "similarity_func": correlation_pearson,
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "disable_warnings": True,
+                "display_progressbar": False,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
@@ -35,8 +66,6 @@ from ...quantus.helpers.model_interface import ModelInterface
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": False,
             },
@@ -51,13 +80,26 @@ from ...quantus.helpers.model_interface import ModelInterface
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Gradient",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": False,
             },
             {"min": -1.0, "max": 1.0},
         ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model_tf"),
+            lazy_fixture("almost_uniform_1d_no_abatch_channel_last"),
+            {
+                "layer_order": "bottom_up",
+                "similarity_func": correlation_pearson,
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Gradient",
+                "disable_warnings": True,
+                "display_progressbar": False,
+                "a_batch_generate": False,
+            },
+            {"exception": ValueError},
+        ),
         (
             lazy_fixture("load_mnist_model_tf"),
             lazy_fixture("load_mnist_images_tf"),
@@ -67,14 +109,26 @@ from ...quantus.helpers.model_interface import ModelInterface
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Gradient",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": False,
                 "a_batch_generate": False,
             },
             {"min": -1.0, "max": 1.0},
         ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "layer_order": "top_down",
+                "similarity_func": correlation_spearman,
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "disable_warnings": True,
+                "display_progressbar": True,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
@@ -84,8 +138,6 @@ from ...quantus.helpers.model_interface import ModelInterface
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": True,
             },
@@ -113,6 +165,18 @@ def test_model_parameter_randomisation(
         )
     else:
         a_batch = None
+
+    if "exception" in expected:
+        with pytest.raises(expected["exception"]):
+            scores_layers = ModelParameterRandomisation(**params)(
+                model=model,
+                x_batch=x_batch,
+                y_batch=y_batch,
+                a_batch=a_batch,
+                **params,
+            )
+        return
+
     scores_layers = ModelParameterRandomisation(**params)(
         model=model,
         x_batch=x_batch,
@@ -136,6 +200,19 @@ def test_model_parameter_randomisation(
 @pytest.mark.parametrize(
     "model,data,params,expected",
     [
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "num_classes": 10,
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "disable_warnings": False,
+                "display_progressbar": False,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
@@ -144,29 +221,52 @@ def test_model_parameter_randomisation(
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": False,
                 "display_progressbar": False,
             },
             {"min": 0.0, "max": 1.0},
         ),
         (
-            lazy_fixture("load_mnist_model"),
-            lazy_fixture("load_mnist_images"),
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "num_classes": 10,
                 "normalise": False,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": False,
                 "a_batch_generate": False,
             },
             {"min": 0.0, "max": 1.0},
         ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "num_classes": 10,
+                "normalise": False,
+                "explain_func": explain,
+                "method": "Saliency",
+                "disable_warnings": True,
+                "display_progressbar": False,
+                "a_batch_generate": False,
+            },
+            {"min": 0.0, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "num_classes": 10,
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "disable_warnings": True,
+                "display_progressbar": True,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
@@ -175,8 +275,6 @@ def test_model_parameter_randomisation(
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "img_size": 28,
-                "nr_channels": 1,
                 "disable_warnings": True,
                 "display_progressbar": True,
             },
@@ -215,6 +313,5 @@ def test_random_logit(
     if isinstance(expected, float):
         assert all(s == expected for s in scores), "Test failed."
     else:
-        assert all(
-            ((s > expected["min"]) & (s < expected["max"])) for s in scores
-        ), "Test failed."
+        assert all(s > expected["min"] for s in scores), "Test failed."
+        assert all(s < expected["max"] for s in scores), "Test failed."
