@@ -1,27 +1,29 @@
-import pytest
+import warnings
+import pickle
 from typing import Union
+
+import numpy as np
+import pytest
 import torch
 import torchvision
-import pickle
 from pytest_lazyfixture import lazy_fixture
+
 from .fixtures import *
 from ..quantus import *
+from ..quantus.helpers.explanation_func import explain
 from ..quantus.helpers.pytorch_model import PyTorchModel
 
-# from ..quantus.helpers import *
-# from ..quantus.metrics import *
-
 
 @pytest.mark.evaluate_func
 @pytest.mark.parametrize(
-    "params,expected",
+    "model,data,params,expected",
     [
         (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
             {
                 "perturb_radius": 0.2,
                 "nr_samples": 10,
-                "img_size": 28,
-                "nr_channels": 1,
                 "explain_func": explain,
                 "method": "Saliency",
                 "disable_warnings": True,
@@ -33,10 +35,26 @@ from ..quantus.helpers.pytorch_model import PyTorchModel
             {"min": 0.0, "max": 1.0},
         ),
         (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_radius": 0.2,
+                "nr_samples": 10,
+                "explain_func": explain,
+                "method": "Saliency",
+                "disable_warnings": True,
+                "normalise": True,
+                "normalise_func": normalise_by_max,
+                "eval_metrics": "{'max-Sensitivity': MaxSensitivity(**params)}",
+                "eval_xai_methods": "{params['method']: a_batch}",
+            },
+            {"min": 0.0, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
             {
                 "nr_samples": 10,
-                "img_size": 28,
-                "nr_channels": 1,
                 "abs": True,
                 "explain_func": explain,
                 "method": "IntegratedGradients",
@@ -49,11 +67,11 @@ from ..quantus.helpers.pytorch_model import PyTorchModel
             {"min": 0.0, "max": 1.0},
         ),
         (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
             {
                 "perturb_radius": 0.2,
                 "nr_samples": 10,
-                "img_size": 28,
-                "nr_channels": 1,
                 "explain_func": explain,
                 "method": "Saliency",
                 "disable_warnings": True,
@@ -65,6 +83,8 @@ from ..quantus.helpers.pytorch_model import PyTorchModel
             {"min": 0.0, "max": 1.0},
         ),
         (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
             {
                 "perturb_radius": 0.2,
                 "nr_samples": 10,
@@ -80,11 +100,11 @@ from ..quantus.helpers.pytorch_model import PyTorchModel
             {"min": 0.0, "max": 1.0},
         ),
         (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
             {
                 "perturb_radius": 0.2,
                 "nr_samples": 10,
-                "img_size": 28,
-                "nr_channels": 1,
                 "explain_func": explain,
                 "method": "InputXGradient",
                 "disable_warnings": True,
@@ -95,11 +115,11 @@ from ..quantus.helpers.pytorch_model import PyTorchModel
             {"min": -1.0, "max": 1.0},
         ),
         (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
             {
                 "perturb_radius": 0.2,
                 "nr_samples": 10,
-                "img_size": 28,
-                "nr_channels": 1,
                 "explain_func": explain,
                 "method": "Gradient",
                 "disable_warnings": True,
@@ -112,16 +132,12 @@ from ..quantus.helpers.pytorch_model import PyTorchModel
     ],
 )
 def test_evaluate_func(
+    model,
+    data: np.ndarray,
     params: dict,
     expected: Union[float, dict, bool],
-    load_mnist_images,
-    load_mnist_model,
 ):
-    model = load_mnist_model
-    x_batch, y_batch = (
-        load_mnist_images["x_batch"],
-        load_mnist_images["y_batch"],
-    )
+    x_batch, y_batch = data["x_batch"], data["y_batch"]
     explain = params["explain_func"]
     a_batch = explain(
         model=model,
@@ -129,12 +145,13 @@ def test_evaluate_func(
         targets=y_batch,
         **params,
     )
+    
     if "exception" in expected:
         with pytest.raises(expected["exception"]):
             results = evaluate(
                 metrics=eval(params["eval_metrics"]),
                 xai_methods=eval(params["eval_xai_methods"]),
-                model=load_mnist_model,
+                model=model,
                 x_batch=x_batch,
                 y_batch=y_batch,
                 a_batch=a_batch,
@@ -146,7 +163,7 @@ def test_evaluate_func(
     results = evaluate(
         metrics=eval(params["eval_metrics"]),
         xai_methods=eval(params["eval_xai_methods"]),
-        model=load_mnist_model,
+        model=model,
         x_batch=x_batch,
         y_batch=y_batch,
         a_batch=a_batch,
